<?php
/**
 * Index database for options, thumbnails, slides, and such.
 *
 * This provides support for managing the index work areas for mod_autoindex.
 * Creation of index database directory is delayed until necessary.
 */

include_once dirname(__FILE__).'/config.php';
include_once dirname(__FILE__).'/class.Img.php';
include_once dirname(__FILE__).'/lib.common.php';

class IndexDb {
	private $index_dir_name = '.index';
	private $thumbs_dir_name = 'thumbs';
	private $slides_dir_name = 'slides';
	private $images_dir_name = 'images';
	private $options_name = 'options';

	private $dir = null;	// absolute path 
	private $dirdb = null;	// database directory for $dir
	private $options = null;
	private $options_dirty = false;

	public function __construct($dir = null) {
		// Note: this class is constructed for directories that are usually from one of two sources.
		// Either curr_dir(), the current directory derived from the current url (good for index pages, bad for ajax)
		// or the saved current directory in $_SESSION['curr_dir'] is used (bad for index pages, good for ajax).
		if ($dir === null) $dir = curr_dir();
		$this->dir = clean_path($dir);

		$index_dir_name = $GLOBALS['INDEXDB_NAME'];	// Set to config setting
	}

	public function __destruct() {
		if ($this->options_dirty) $this->save_options();
	}

	public function option($opt,$val=null) {
		if (! $this->options) $this->get_options();
		if ($val !== null) { $this->options[$opt] = $val; $this->options_dirty = true; }
		return $this->options[$opt];
	}

	public function header($html=null) {
		$path = $this->dirdb_path() . DIRECTORY_SEPARATOR . 'header.html';
		if ($html !== null) {
			if ($this->dirdb === null) $this->get_dirdb();
			$fd = fopen($path,'w');
			if ($fd) {
				fwrite($fd,$html);
				fclose($fd);
			}
			else {
				warn("Cannot save header to file $path.");
			}
		}
		return is_readable($path) ? file_get_contents($path) : '';
	}

	public function footer($html=null) {
		$path = $this->dirdb_path() . DIRECTORY_SEPARATOR . 'footer.html';
		if ($html !== null) {
			if ($this->dirdb === null) $this->get_dirdb();
			$fd = fopen($path,'w');
			if ($fd) {
				fwrite($fd,$html);
				fclose($fd);
			}
			else {
				warn("Cannot save footer to file $path.");
			}
		}
		return is_readable($path) ? file_get_contents($path) : '';
	}

	public function save() {
		// Save index database.
		// Currently this means only saving options as the other info is saved as it is set.
		// This is an optional function. The index database is intended to be thought of a database
		// where transactions are automatically saved. It ensures everything is saved and not cached.
		// However, in dialogs it's advantagous to return confirmation that the index database can be saved.
		if ($this->save_options()) {
			$this->options_dirty = false;
			return true;
		}
		return false;
	}

	public function thumb($src) {
		$path = $this->thumb_path($src);
		if (! is_readable($path) || filemtime($path) < filemtime($src)) $this->make_thumb($src);
		return is_readable($path) ? file_get_contents($path) : $this->thumb_404();
	}
	public function thumb_mime_type($src) {
		$path = $this->thumb_path($src);
		if (! is_readable($path)) $this->make_thumb($src);
		// Note: According to PHP GD intro, getimagesize() does not require GD extension.
		$info = getimagesize($path);
		if ($info) {
			return $info['mime'];
		} else {
			return 'image/png';	// mime-type for empty string returned for non-images
		}
	}
	public function thumb_path($src) {
		// Return name of file used for thumbnail of image file.
		if ($this->dirdb === null) $this->get_dirdb();
		return $this->dirdb . DIRECTORY_SEPARATOR . $this->thumbs_dir_name . DIRECTORY_SEPARATOR . basename($src);
	}
	public function thumb_404() {
		return file_get_contents($GLOBALS['MYRA_THEMES_PATH'].'/'.my_theme().'/images/thumb-404.png');
	}

	public function slide($src) {
		$path = $this->slide_path($src);
		if (! is_readable($path) || filemtime($path) < filemtime($src)) $this->make_slide($src);
		return is_readable($path) ? file_get_contents($path) : '';
	}
	public function slide_mime_type($src) {
		$path = $this->slide_path($src);
		if (! is_readable($path)) $this->make_slide($src);
		// Note: According to PHP GD intro, getimagesize() does not require GD extension.
		$info = getimagesize($path);
		if ($info) {
			return $info['mime'];
		} else {
			return 'image/jpg';	// mime-type for empty string returned for non-images
		}
	}
	public function slide_path($src) {
		// Return name of file used for slide of image file.
		if ($this->dirdb === null) $this->get_dirdb();
		return $this->dirdb . DIRECTORY_SEPARATOR . $this->slides_dir_name . DIRECTORY_SEPARATOR . basename($src);
	}

	public function image($src) {
		$path = $this->image_path($src);
		if (! is_readable($path) || filemtime($path) < filemtime($src)) $this->make_image($src);
		return is_readable($path) ? file_get_contents($path) : '';
	}
	public function image_mime_type($src) {
		$path = $this->image_path($src);
		if (! is_readable($path)) $this->make_image($src);
		// Note: According to PHP GD intro, getimagesize() does not require GD extension.
		$info = getimagesize($path);
		if ($info) {
			return $info['mime'];
		} else {
			return 'image/jpg';	// mime-type for empty string returned for non-images
		}
	}
	public function image_path($src) {
		// Return name of file used for image of image file.
		if ($this->dirdb === null) $this->get_dirdb();
		return $this->dirdb . DIRECTORY_SEPARATOR . $this->images_dir_name . DIRECTORY_SEPARATOR . basename($src);
	}

	public function rotate($src,$deg) {
		// Rotate thumbnails and slides but not original.
		$path = $this->image_path($src);
		if (! is_readable($path) || filemtime($path) < filemtime($src)) $this->make_image($src);

		$img = new Img($path);
		$img->rotate($deg);
		$img->save($path);

		$img->resize($GLOBALS['SLIDE_WIDTH'],$GLOBALS['SLIDE_HEIGHT']);
		$img->save($this->slide_path($src));

		$img = new Img($path);
		$img->thumbnail($GLOBALS['THUMB_WIDTH'],$GLOBALS['THUMB_HEIGHT']);
		$img->save($this->thumb_path($src));
	}

	public function refresh($src) {
		// Remake image, slide, and thumbnail from original
		$this->make_image($src);
		$this->make_slide($src);
		$this->make_thumb($src);
	}

	public function editors() {
		// List usernames of people that can edit current directory.
		// Note: this does not check if the people can access current directory.
		$editors = array();

		preg_match_all('/^\\s*((editors\\w*?)\\s*:\\s*(.*))\\s*/m',file_get_contents($GLOBALS['GROUP_FILE']),$matches);
		preg_match_all('/^\\s*(editors.*)\\s*$/m',file_get_contents($GLOBALS['GROUP_FILE']),$matches);
		$editors_lines = $matches[1];

		foreach ($editors_lines as $line) {
			// Directory where editors are enabled is embedded in group name.
			$editors_dir = clean_path($GLOBALS['DOCUMENT_ROOT'] . preg_replace('/-/','/',preg_replace('/\\s*:.*/','',preg_replace('/^\\s*editors([-\\w]*)/','$1',$line))));
			if (strpos($this->dir.'/',$editors_dir.'/') === 0) {
				$usernames = preg_split('/ /',preg_replace('/.*:\\s*/','',$line));
				$editors = array_merge($editors,$usernames);
			}
		}
		sort($editors);
		return $editors;
	}

	public function remove($filename) {
		// Remove info for $filename from the index db.
		if (file_exists($this->thumb_path($filename))) unlink($this->thumb_path($filename));
		if (file_exists($this->slide_path($filename))) unlink($this->slide_path($filename));
		if (file_exists($this->image_path($filename))) unlink($this->image_path($filename));
	}

	public function rename($filename,$newname) {
		// Rename file in index db (not original file).
		if (file_exists($this->thumb_path($filename))) { chdir(dirname($this->thumb_path($filename))) and rename($filename,$newname); }
		if (file_exists($this->slide_path($filename))) { chdir(dirname($this->slide_path($filename))) and rename($filename,$newname); }
		if (file_exists($this->image_path($filename))) { chdir(dirname($this->image_path($filename))) and rename($filename,$newname); }
	}

	//
	// Private methods
	//

	private function dirdb_path() {
		// The index database is either in its own location or in the same directory being indexed.
		// Note: If the database directory for $this->dir is not to be located
		// in $this->dir, this is the place to change its actual location. Just
		// substitute the DocumentRoot portion of $this->dir with a different directory path.
		if ($GLOBALS['INDEXDB']) {
			$dir = $GLOBALS['INDEXDB'] . preg_replace('|'.$GLOBALS['DOCUMENT_ROOT'].'|','',$this->dir);
		}
		else {
			$dir = $this->dir;	// put index db in same directory being indexed
		}
		return $dir . DIRECTORY_SEPARATOR . $this->index_dir_name;
	}

	private function get_dirdb() {
		// Get index database (directory)
		$this->dirdb = $this->dirdb_path();
		if (! is_dir($this->dirdb)) $this->make_dirdb();
		if (! is_dir($this->dirdb)) {
			// Prevent any file operations from performing if the database directory cannot be accessed.
			//return false;
			trigger_error("Cannot create or find the index database directory {$this->dirdb}.",E_USER_ERROR);
		}
		return $this->dirdb;
	}
	private function make_dirdb() {
		// Make a directory .index in the target directory.
		static $done = null;
		if ($done !== null) return $done;	// prevent multiple failed attempts

		$dir = escapeshellarg($this->dir);
		system("{$GLOBALS['MYRA_SUPPORT_PATH']}/scripts/make-indexdb $dir",$ret);
		if ($ret) {
			$s = $ret == 1 ? ' (Are SUDO privileges needed?)' : '';
			warn("Cannot make index database for directory: {$this->dir}. (errno $ret)$s");
			$done = false;
			return $done;
		}
		$done = true;
		return $done;
	}
	private function make_thumb($src) {
		if ($this->dirdb === null) { if (! $this->get_dirdb()) return false; }
		// Create thumbnail
		$img = new Img($src);
		$img->thumbnail($GLOBALS['THUMB_WIDTH'],$GLOBALS['THUMB_HEIGHT']);
		$rc = $img->save($this->thumb_path($src)) or warn("Cannot make thumbnail for $src in ".$this->thumb_path($src)." ($rc)");
		return $rc;
	}
	private function make_slide($src) {
		if ($this->dirdb === null) { if (! $this->get_dirdb()) return false; }
		// Create slide
		$img = new Img($src);
		$img->resize($GLOBALS['SLIDE_WIDTH'],$GLOBALS['SLIDE_HEIGHT']);
		$rc = $img->save($this->slide_path($src)) or warn("Cannot make slide for $src in ".$this->slide_path($src)." ($rc)");
		return $rc;
	}
	private function make_image($src) {
		if ($this->dirdb === null) { if (! $this->get_dirdb()) return false; }
		// Create image
		$img = new Img($src);
		$img->resize($GLOBALS['IMAGE_WIDTH'],$GLOBALS['IMAGE_HEIGHT'],2);
		$rc = $img->save($this->image_path($src)) or warn("Cannot make image for $src in ".$this->image_path($src)." ($rc)");
		return $rc;
	}
	private function get_options() {
		if (! is_readable($this->options_path())) return false;
		return $this->options = json_decode(file_get_contents($this->options_path()),true);
	}
	private function save_options() {
		$this->get_dirdb();	// ensure database dir is created
		$path = $this->options_path();
		$fd = fopen($path,'w');
		if ($fd) {
			fwrite($fd,json_encode($this->options));
			fclose($fd);
			return true;
		}
		else {
			warn("Cannot save options to file $path.");
			return false;
		}
	}
	private function options_path() {
		// Return name of file used for options
		return $this->dirdb_path() . DIRECTORY_SEPARATOR . $this->options_name;
	}
}
?>
