<?php
/**
 * User Authentication / Authorization Functions
 *
 * Usage:
 * Functions are to be used at the beginning of programs.
 *
 *	$user = authenticate_user($name,$pass) or die_unauthenticated();
 *	 -or-
 *	$user = authorize_user($group) or die_unauthorized($group);
 *	 -or-
 *	$user = authenticate_user($name,$pass) or exit_login();
 *	 -or-
 *	$user = authorize_user('mygroup') or exit_login('mygroup');
 *
 * The returned value $user is a User object or false.
 *
 * Additional authentication functions:
 *
 *	$bool = can_make_file_in_current_dir();
 *	$bool = can_upload_to_current_dir();
 *	$bool = can_upload($dir,$user);
 *	$bool = can_rearrange_current_dir();
 *	$bool = can_rearrange();
 *	$bool = can_edit_current_dir();
 *	$bool = can_edit_current_file();
 *	$bool = can_edit($file,$user);
 */

include_once dirname(__FILE__).'/config.php';
include_once dirname(__FILE__).'/class.Session.php';
include_once dirname(__FILE__).'/class.User.php';
include_once dirname(__FILE__).'/lib.common.php';

// Uncomment the next to test this without having to exit browser in order to clear session variables.
//session_stop();	//### TODO: remove when done testing
//session_start();
//$_SESSION['user'] = new User($_SERVER['PHP_AUTH_USER']);

function authenticate_user($name=null,$pass=null) {
	$session = Session::getInstance() or exit_error('Cannot initiate a new session.');

	debug('authenticate_user: name='.($name == null ? 'NULL'.', PHP_AUTH_USER="'.(isset($_SERVER['PHP_AUTH_USER']) ? $_SERVER['PHP_AUTH_USER'] : 'NOT SET').'"' : $name));

	// The presence of $_SERVER['PHP_AUTH_USER'] means that the current PATH
	// is protected and the username is provided by the browser to access the PATH.
	// Note: PHP_AUTH_USER is not set unless the path is protected.
	if ($name == null && isset($_SERVER['PHP_AUTH_USER'])) {
		$name = $_SERVER['PHP_AUTH_USER'];
		$pass = $_SERVER['PHP_AUTH_PW'];
	}

	if ($name) {
		unset($session->authenticated_user);
	}

	// Note: As authentication token is stored in the browser, the web server will allow access to any
	// of this user's documents but the following should shut down anything going through Myra.
	debug('authenticate_user: logged_out='.($session->logged_out ? $session->logged_out : 'UNDEFINED'));
	if (isset($session->logged_out) && $session->logged_out && ! preg_match('|/signout.php$|',$_SERVER['PHP_SELF'])) {
		// Redirect to signout page
		exit_signout();
	}

	if (isset($session->user)) {
		if ($name && ! $session->user->username()) {
			// First time authentication name passed to us from Apache.
			// Redefine User object to use given name.
			$session->user = new User($name);
		}
		if ($name && $session->user->username() != $name) {
			// The user name is being changed which effectivily cannot be done with Apache authentication.
			security('User name being changed from "'.$session->user->username().'" to "'.$name.'"');
			return false;
		}
	}
	else {
		$session->user = new User($name);
	}

	// Note: if $name is empty here, it denotes we are on a page that is not protected as
	// $_SERVER['PHP_AUTH_USER'] is not set either which php sets only for protected pages.

	if ($name) {
		// Authorize browser before authenticating user. 
		// Note: this is safer and better done with rewrite rules in /.htaccess.
		is_approved_browser() || exit_bad_browser();

		if (! $session->user->authenticate($pass)) return false;

		// Set name used for display in buttons at top of page.
		$session->authenticated_user = $name;
	}
	

	// Put username into environment for support shell scripts
	putenv("REMOTE_USER=".$session->user->username());
	$GLOBALS['REMOTE_USER'] = $session->user->username();

	debug('authenticate_user: AUTHENTICATED: name='.$name.', PHP_AUTH_USER='.(isset($_SERVER['PHP_AUTH_USER']) ? $_SERVER['PHP_AUTH_USER'] : 'UNDEFINED'));

	return $session->user;
}

function die_unauthenticated($msg = null) {
	if ($msg == null) $msg = authentication_error();
	warn($msg);
	exit_error($msg);
}

function authorize_user($name=null,$pass=null,$group=null) {
	$user = authenticate_user($name,$pass);
	if (! $user) return false;
	if (! $user->authorize($group)) return false;
	return $user;
}

function die_unauthorized($msg = null) {
	if ($msg == null) $msg = authorization_error();
	warn($msg);
	exit_error($msg);
}

function authentication_error() {
	return $GLOBALS['AUTHENTICATION_ERROR'] ? $GLOBALS['AUTHENTICATION_ERROR'] : 'You cannot be authenticated at this time.';
}

function authorization_error() {
	return $GLOBALS['AUTHORIZATION_ERROR'] ? $GLOBALS['AUTHORIZATION_ERROR'] : 'You are not authorized.';
}

/*
 * Browser approval
 */

function is_approved_browser($ua = null) {
	if ($ua == null) $ua = $_SERVER['HTTP_USER_AGENT'];

	// Special Approvals
	// Uncomment following to test browsers.
	/***
	if ($_SERVER['PHP_AUTH_USER'] == 'mack') {
		// User used for testing.
		debug('is_approved_browser: true, user==mack');
		return true;
	}
	***/

	// Reject unapproved browsers
	$browser = browser($ua);
	switch ($browser) {
	case '':	return false;	// browser unknown
	case 'android':	return false;	// does not log out
	case 'opera':	return false;	// not tested	TODO: test this
	}

	debug('is_approved_browser: '.$browser.' APPROVED, _SERVER[HTTP_USER_AGENT]='.$_SERVER['HTTP_USER_AGENT']);
	return true;
}

function exit_bad_browser($msg='') {
	$session = Session::getInstance();
	$session->msg = $msg;
	$msg = $msg ? $msg : 'This browser is not approved for this system. Please use a different browser.';
	trigger_error($msg . backtrace(10),E_USER_WARNING);	// use E_USER_WARNING to avoid death before redirect

	if (is_ajax_request()) {
		exit_result(false,$msg);
	}
	elseif (is_myra_dialog()) {
		exit_dialog_error($msg);
	}
	else {
		# Following does not work for index page (no headers processed)
		#exit_next($GLOBALS['BAD_BROWSER_URL']);
		# Cannot prevent file list on first list. Hide it from display. (!!)
		print <<<End_Of_HTML
<html><head><meta http-equiv="refresh" content="0; url={$GLOBALS['BAD_BROWSER_URL']}"><style>body{display:none}</style></head><body></body></html>
End_Of_HTML;
		exit;
	}
}

function exit_signout($msg='') {
	$session = Session::getInstance();
	$session->msg = $msg;
	$msg = $msg ? $msg : 'You have not completed the logout process yet. Please exit this browser.';

	if (is_ajax_request()) {
		exit_result(false,$msg);
	}
	elseif (is_myra_dialog()) {
		exit_dialog_error($msg);
	}
	else {
		# Following does not work for index page (no headers processed)
		#exit_next($GLOBALS['SIGNOUT_URL']);
		# Cannot prevent file list on first list. Hide it from display. (!!)
		print <<<End_Of_HTML
<html><head><meta http-equiv="refresh" content="0; url={$GLOBALS['SIGNOUT_URL']}"><style>body{display:none}</style></head><body></body></html>
End_Of_HTML;
		exit;
	}
}

/*
 * File privileges authorization.
 *
 * Note: the functions dealing with the current directory or current file are only
 * valid after the context has been saved (inc/myra_preamble.php) or switched.
 */

function can_make_file_in_current_dir() {
	static $cache_can_make = null;
	if ($cache_can_make != null) return $cache_can_make;
	$cache_can_make = can_edit_current_dir();
	return $cache_can_make;
}

function can_upload_to_current_dir() {
	static $cache_can_upload = null;
	if ($cache_can_upload != null) return $cache_can_upload;
	$session = Session::getInstance();
	$cache_can_upload = can_upload($session->curr_dir,$session->user);
	return $cache_can_upload;
}

function can_upload($dir,$user) {
	if (can_edit($dir,$user)) return true;
	include_once dirname(__FILE__).'/class.IndexDb.php';
	global $idb;
	if (!isset($idb)) $idb = new IndexDb($dir);
	return $idb->option('guest_can_upload') ? true : false;
}

function can_rearrange_current_dir() {
	static $cache_rearrange = null;
	if ($cache_rearrange != null) return $cache_rearrange;
	$session = Session::getInstance();
	$cache_rearrange = can_rearrange($session->curr_dir,$session->user);
	return $cache_rearrange;
}
function can_rearrange($dir,$user) {
	if (can_edit($dir,$user)) return true;
	include_once dirname(__FILE__).'/class.IndexDb.php';
	global $idb;
	if (!isset($idb)) $idb = new IndexDb($dir);
	return $idb->option('guest_can_sort') ? true : false;
}

function can_edit_current_dir() {
	static $cache_can_edit_dir = null;
	if ($cache_can_edit_dir != null) return $cache_can_edit_dir;
	$session = Session::getInstance();
	$cache_can_edit_dir = can_edit($session->curr_dir,$session->user);
	return $cache_can_edit_dir;
}

function can_edit_current_file() {
	static $cache_can_edit = null;
	if ($cache_can_edit != null) return $cache_can_edit;
	$session = Session::getInstance();
	$current_file = $session->curr_file ? $session->curr_dir.DIRECTORY_SEPARATOR.$session->curr_file : $session->curr_dir;
	$cache_can_edit = $current_file && can_edit($current_file,$session->user);
	return $cache_can_edit;
}

function can_edit($file=null,$user=null) {
	if ($file === null) $file = realpath($GLOBALS['DOCUMENT_ROOT'].preg_replace('/\?.*/','',$_SERVER['REQUEST_URI']));
	if (! file_exists($file)) return false;		// file does not exist on this server (a local file)
	if (preg_match('|'.realpath($GLOBALS['DOCUMENT_ROOT'].$GLOBALS['MYRA']).'|',$file)) {
		// disallow editing of Myra programs.
		return false;
	}
	if (! $GLOBALS['ESCALATE_EDITOR_PRIVILEGES'] && ! is_writeable($file)) return false;
	if ($user === null) {
		$session = Session::getInstance();
		$user = $session->user;
		if (!$user) return false;
	}
	return $user->is_editor($file);
}

function can_delete($file=null,$user=null) {
	return can_edit($file,$user);
}

function can_delete_current_file() {
	static $cache_can_delete = null;
	if ($cache_can_delete != null) return $cache_can_delete;
	$session = Session::getInstance();
	$current_file = $session->curr_file ? $session->curr_dir.DIRECTORY_SEPARATOR.$session->curr_file : $session->curr_dir;
	$cache_can_delete = $current_file && can_delete($current_file,$session->user);
	return $cache_can_delete;
}

/*
 * Note: The following routines are useful for Cookie based security and not when Apache authentication is used.
 * The following globals are used here that would need to be in config.php if used.
 *   $GLOBALS['LOGIN_GROUPS']
 *   $GLOBALS['GROUP_HOME_URL']
 *   $GLOBALS['AUTHENTICATION_ERROR']
 *   $GLOBALS['AUTHORIZATION_ERROR']
 *   $GLOBALS['LOGIN_URL']
 */

/********************************************************************************
function exit_login($group = '',$msg = '', $return_url = '') {
	// Display login form.
	if ($group && ! $return_url) $return_url = $_SERVER['REQUEST_URI'];
	$session = Session::getInstance();
	$session->error_msg = $msg;
	$session->return_url = $return_url;
	exit_next(add_querystring_arg(login_url(),"group=$group"));
}

function exit_authorize_user($username,$password,$group=null) {
	// User logging in from login form.
	$session = Session::getInstance();
	$session->destroy();

	if (! $return_url) $return_url = isset($_REQUEST['url']) ? $_REQUEST['url'] : '';

	$user = new User($username);

	if (! $user->is_valid()) exit_login($group,"Your user name is not recognized.",$return_url);	// repeat

	// Pick out the first of the standard login groups that the user is authorized.
	$auth_group = $group;
	if (! $auth_group && is_array($GLOBALS['LOGIN_GROUPS'])) {
		for ($i=0; $i < count($GLOBALS['LOGIN_GROUPS']); $i++) {
			if ($user->is_authorized($GLOBALS['LOGIN_GROUPS'][$i])) {
				$auth_group = $GLOBALS['LOGIN_GROUPS'][$i];
				break;
			}
		}
	}

	if (! $user->is_authorized($password,$auth_group)) exit_login($group,"Your user name or password is not recognized.",$return_url);	// repeat

	// Success
	// Reinitialize session.
	$session = Session::getInstance();

	// Save user and group in session.
	$session->user = $user;
	$session->group = $auth_group;

	if (! $return_url) {
		$return_url = isset($GLOBALS['GROUP_HOME_URL'][$auth_group]) ? $GLOBALS['GROUP_HOME_URL'][$auth_group] : "{$GLOBALS['MYRA']}/";
	}

	// Return to url originating login if known or to group home page.
	exit_next($return_url);
}

function login_url() {
	return $GLOBALS['LOGIN_URL'] ? $GLOBALS['LOGIN_URL'] : $GLOBALS['MYRA'].'/signin.php';
}
********************************************************************************/
